@@ -181,7 +181,11 @@ class DFMessage(object):
         except Exception:
             raise AttributeError(field)
         if isinstance(self._elements[i], bytes):
-            v = self._elements[i].decode("utf-8")
+            try:
+                v = self._elements[i].decode("utf-8")
+            except UnicodeDecodeError:
+                # try western europe
+                v = self._elements[i].decode("ISO-8859-1")
         else:
             v = self._elements[i]
         if self.fmt.format[i] == 'a':
@@ -242,11 +246,13 @@ class DFMessage(object):
             if is_py2:
                 if isinstance(v,unicode): # NOQA
                     v = str(v)
+                elif isinstance(v, array.array):
+                    v = v.tostring()
             else:
                 if isinstance(v,str):
                     v = bytes(v,'ascii')
-            if isinstance(v, array.array):
-                v = v.tostring()
+                elif isinstance(v, array.array):
+                    v = v.tobytes()
             if mul is not None:
                 v /= mul
                 v = int(round(v))
