@@ -452,29 +452,57 @@ def pitch_sim(SIMSTATE, GPS_RAW):
         return -0
     return degrees(-asin(xacc/zacc))
 
-def distance_two(GPS_RAW1, GPS_RAW2, horizontal=True):
-    '''distance between two points'''
-    if hasattr(GPS_RAW1, 'Lat'):
-        lat1 = radians(GPS_RAW1.Lat)
-        lat2 = radians(GPS_RAW2.Lat)
-        lon1 = radians(GPS_RAW1.Lng)
-        lon2 = radians(GPS_RAW2.Lng)
-        alt1 = GPS_RAW1.Alt
-        alt2 = GPS_RAW2.Alt
-    elif hasattr(GPS_RAW1, 'cog'):
-        lat1 = radians(GPS_RAW1.lat)*1.0e-7
-        lat2 = radians(GPS_RAW2.lat)*1.0e-7
-        lon1 = radians(GPS_RAW1.lon)*1.0e-7
-        lon2 = radians(GPS_RAW2.lon)*1.0e-7
-        alt1 = GPS_RAW1.alt*0.001
-        alt2 = GPS_RAW2.alt*0.001
+ORGN = None
+
+def get_origin():
+  global ORGN
+  if ORGN is not None:
+      return ORGN
+  from . import mavutil
+  self = mavutil.mavfile_global
+  ret = self.messages.get('ORGN', None)
+  if ret is None:
+      ret = self.messages.get('GPS', None)
+      if ret.Status < 3:
+          return None
+  ORGN = ret
+  return ret
+
+# graph distance_two(GPS,XKF1[0])
+
+def get_lat_lon_alt(MSG):
+    '''gets lat and lon in radians and alt in meters from a position msg'''
+    if hasattr(MSG, 'Lat'):
+        lat = radians(MSG.Lat)
+        lon = radians(MSG.Lng)
+        alt = MSG.Alt
+    elif hasattr(MSG, 'cog'):
+        lat = radians(MSG.lat)*1.0e-7
+        lon = radians(MSG.lon)*1.0e-7
+        alt = MSG.alt*0.001
+    elif hasattr(MSG,'lat'):
+        lat = radians(MSG.lat)
+        lon = radians(MSG.lon)
+        alt = MSG.alt*0.001
+    elif hasattr(MSG, 'PN'):
+        # origin relative position from EKF
+        global ORGN
+        if ORGN is None:
+            ORGN = get_origin()
+        if ORGN is None:
+            return None
+        (lat,lon) = gps_offset(ORGN.Lat, ORGN.Lng, MSG.PN, MSG.PE)
+        lat = radians(lat)
+        lon = radians(lon)
+        alt = ORGN.Alt - MSG.PD
     else:
-        lat1 = radians(GPS_RAW1.lat)
-        lat2 = radians(GPS_RAW2.lat)
-        lon1 = radians(GPS_RAW1.lon)
-        lon2 = radians(GPS_RAW2.lon)
-        alt1 = GPS_RAW1.alt*0.001
-        alt2 = GPS_RAW2.alt*0.001
+        return None
+    return (lat, lon, alt)
+
+def _distance_two(MSG1, MSG2, horizontal=True):
+    '''distance between two points'''
+    (lat1, lon1, alt1) = get_lat_lon_alt(MSG1)
+    (lat2, lon2, alt2) = get_lat_lon_alt(MSG2)
     dLat = lat2 - lat1
     dLon = lon2 - lon1
 
@@ -485,6 +513,13 @@ def distance_two(GPS_RAW1, GPS_RAW2, horizontal=True):
         return ground_dist
     return sqrt(ground_dist**2 + (alt2-alt1)**2)
 
+def distance_two(MSG1, MSG2, horizontal=True):
+    '''distance between two points'''
+    try:
+        return _distance_two(MSG1, MSG2)
+    except Exception as ex:
+        print(ex)
+        return None
 
 first_fix = None
 
