@@ -24,12 +24,13 @@ parser.add_argument("--overlap", dest='fft_overlap', default=False, action='stor
 parser.add_argument("--output", dest='fft_output', default='psd', action='store', help="whether to output frequency spectrum information as power or linear spectral density: 'psd' or 'lsd'")
 parser.add_argument("--notch-params", default=False, action='store_true', help="whether to output estimated harmonic notch parameters")
 parser.add_argument("--notch-peak", dest='fft_peak', default=0, action='store', help="peak to select when setting notch parameters")
+parser.add_argument("--axis", dest='axis', default='XYZ', action='store', help="Select which axis to plot default: XYZ")
 
 args = parser.parse_args()
 
 from pymavlink import mavutil
 
-def mavfft_fttd(logfile):
+def mavfft_fttd(logfile, multi_log):
     '''display fft for raw ACC data in logfile'''
 
     '''object to store data about a single FFT plot'''
@@ -161,7 +162,7 @@ def mavfft_fttd(logfile):
     window = {}
     S2 = {}
     hntch_mode_names = { 0:"No", 1:"Throttle", 2:"RPM", 3:"ESC", 4:"FFT"}
-    hntch_option_names = { 0:"Single", 1:"Double", 2:"Dynamic Harmonic", 3:"Double+Dynamic"}
+    hntch_option_names = { 0:"Single", 1:"Double", 2:"Dynamic", 4:"Loop-Rate"}
     batch_mode_names = { 0:"Pre-filter", 1:"Sensor-rate", 2:"Post-filter" }
     fft_peak = int(args.fft_peak)
 
@@ -192,6 +193,9 @@ def mavfft_fttd(logfile):
             S2[thing_to_plot.tag()] = numpy.inner(window[thing_to_plot.tag()], window[thing_to_plot.tag()])
 
         for axis in [ "X","Y","Z" ]:
+            # only plot the selected axis
+            if axis not in args.axis:
+                continue
             # normalize data and convert to dps in order to produce more meaningful magnitudes
             if thing_to_plot.sensor_type == 1:
                 d = numpy.array(numpy.degrees(thing_to_plot.data[axis])) / float(thing_to_plot.multiplier)
@@ -226,6 +230,10 @@ def mavfft_fttd(logfile):
         print("Sensor: %s" % str(sensor))
         fig = pylab.figure(str(sensor))
         for axis in [ "X","Y","Z" ]:
+            # only plot the selected axis
+            if axis not in args.axis:
+                continue
+
             # normalize output to averaged PSD
             psd = 2 * (sum_fft[sensor][axis] / counts[sensor]) / (sample_rates[sensor] * S2[sensor])
 
@@ -246,7 +254,15 @@ def mavfft_fttd(logfile):
             # convert to db if requested
             if args.fft_scale == 'db':
                 psd = 10 * numpy.log10 (psd)
-            pylab.plot(freqmap[sensor], psd, label=axis)
+            # add file name to axis if doing multi-file plot
+            if multi_log:
+                # remove extension and path
+                (log_name, _, _) = logfile.rpartition('.')
+                (_, _, log_name) = log_name.rpartition('/')
+                legend_label = '%s (%s)' % (axis, log_name)
+            else:
+                legend_label = axis
+            pylab.plot(freqmap[sensor], psd, label=legend_label)
         pylab.legend(loc='upper right')
         pylab.xlabel('Hz')
         scale_label=''
@@ -265,9 +281,15 @@ def mavfft_fttd(logfile):
                 pylab.ylabel('PSD $' + scale_label + 'm^2/s^4/Hz$')
 
         if hntch_mode is not None and hntch_option is not None and batch_mode is not None:
+            option_label = ""
+            for hopt in hntch_option_names:
+                if hopt & int(hntch_option) != 0:
+                    if len(option_label) > 0:
+                        option_label += "+"
+                    option_label += hntch_option_names[hopt]
             textstr = '\n'.join((
                 r'%s tracking' % (hntch_mode_names[hntch_mode], ),
-                r'%s notch' % (hntch_option_names[hntch_option], ),
+                r'%s notch' % (option_label, ),
                 r'%s sampling' % (batch_mode_names[batch_mode], )))
 
             props = dict(boxstyle='round', facecolor='wheat', alpha=0.5)
@@ -275,7 +297,12 @@ def mavfft_fttd(logfile):
             pylab.text(0.5, 0.95, textstr, fontsize=12,
                 verticalalignment='top', bbox=props, transform=pylab.gca().transAxes)
 
+# auto set option for adding log names to legend label
+multi_log = False
+if len(args.logs) > 1:
+    multi_log = True
+
 for filename in args.logs:
-    mavfft_fttd(filename)
+    mavfft_fttd(filename, multi_log)
 
 pylab.show()
